@@ -3,7 +3,7 @@ package Clustericious::Client;
 use strict; no strict 'refs';
 use warnings;
 
-our $VERSION = '0.73';
+our $VERSION = '0.74';
 
 =head1 NAME
 
@@ -339,7 +339,7 @@ sub route {
         no strict 'refs';
         *{caller() . "::$subname"} = sub {
             my $self = shift;
-            my @args = $self->meta_for->process_args(@_);
+            my @args = $self->meta_for($subname)->process_args(@_);
             my $got = $self->_doit($meta,$method,$url,@args);
             return $objclass->new($got, $self) if $objclass;
             return $got;
@@ -598,48 +598,43 @@ sub _doit {
 
     $url = Mojo::URL->new($url) unless ref $url;
     my $parameters = $url->query;
+
+    # Set up mappings from parameter names to modifier callbacks.
     my %url_modifier;
     my %payload_modifer;
     my %gen_url_modifier = (
-        query => sub { my $name = shift;  sub { my ($u,$v) = @_; $u->query({$name => $v}) }  },
-        append => sub { my $name = shift; sub { my ($u,$v) = @_; push @{ $u->path->parts } , $v; $u; } },
+        query  => sub { my $name = shift;
+            sub { my ($u,$v) = @_; $u->query({$name => $v}) }  },
+        append => sub { my $name = shift;
+            sub { my ($u,$v) = @_; push @{ $u->path->parts } , $v; $u; } },
+    );
+    my %gen_payload_modifier = (
+        array => sub {
+            my ( $name, $key ) = @_;
+            LOGDIE "missing key for array payload modifier" unless $key;
+            sub { my $body = shift; $body ||= {}; push @{ $body->{$key} }, ( $name => shift ); $body; }
+        },
+        hash => sub {
+            my $name = shift;
+            sub { my $body = shift; $body ||= {}; $body->{$name} = shift; $body; }
+        },
     );
-    my @positional_args;
     if ($meta && (my $arg_spec = $meta->get('args'))) {
-        my @new_args;
-        my %a;
-        %a = @args unless @args % 2;
         for (@$arg_spec) {
-            push @positional_args, $_->{name} if $_->{positional} && $_->{positional} eq 'one';
+            my $name = $_->{name};
             if (my $modifies_url = $_->{modifies_url}) {
-                if (ref ($modifies_url) eq 'CODE') {
-                    $url_modifier{$_->{name}} = $modifies_url;
-                } elsif (my $gen = $gen_url_modifier{$modifies_url}) {
-                    $url_modifier{$_->{name}} = $gen->($_->{name});
-                } else  {
-                    die "don't understand how to interepret modifies_url=$modifies_url";
-                }
+                $url_modifier{$name} =
+                    ref($modifies_url) eq 'CODE'            ? $modifies_url
+                 :  ($a = $gen_url_modifier{$modifies_url}) ? $a->($name)
+                 :  die "don't understand how to interpret modifies_url=$modifies_url";
             }
-            my $modifies_payload = $_->{modifies_payload} || "";
-            if ($modifies_payload eq 'array') {
-                my $name = $_->{name};
-                my $key = $_->{key} or LOGDIE "No key for $name (modifies_payload=array needs a hash key to act on)";
-                $payload_modifer{$name} = sub { my $body = shift; $body ||= {}; push @{ $body->{$key} }, ( $name => shift); $body };
-            }
-            if ($modifies_payload eq 'hash') {
-                my $name = $_->{name};
-                $payload_modifer{$name} = sub { my $body = shift; $body ||= {}; $body->{$name} = shift; $body };
-            }
-
-            if ($_->{positional} && @args) {
-                push @new_args, shift @args;
-                %a = @args unless @args % 2;
-            }
-            if (!$_->{positional}) {
-                push @new_args, ($_->{name} => $a{$_->{name}}) if exists($a{$_->{name}});
+            if (my $modifies_payload = $_->{modifies_payload}) {
+                 $payload_modifer{$name} =
+                    ref($modifies_payload) eq 'CODE' ? $modifies_payload
+                 : ($a = $gen_payload_modifier{$modifies_payload}) ? $a->($name,$_->{key})
+                 : LOGDIE "don't understand how to interpret modifies_payload=$modifies_payload";
             }
         }
-        @args = @new_args;
     }
 
     while (defined(my $arg = shift @args)) {
@@ -650,9 +645,6 @@ sub _doit {
             $headers = { 'Content-Type' => 'application/json' };
         } elsif (ref $arg eq 'CODE') {
             $cb = $self->_mycallback($arg);
-        } elsif (@positional_args && (my $poscode = $url_modifier{$positional_args[0]})) {
-            $url = $poscode->($url, $arg);
-            shift @positional_args;
         } elsif (my $code = $url_modifier{$arg}) {
             $url = $code->($url, shift @args);
         } elsif (my $code2 = $payload_modifer{$arg}) {
